# Generated by extendr: Do not edit by hand
#
# This file was created with the following call:
#   .Call("wrap__make_changeforestr_wrappers", use_symbols = TRUE, package_name = "changeforest")

#' @docType package
#' @usage NULL
#' @useDynLib changeforest, .registration = TRUE
NULL

changeforest_api <- function(X, method, segmentation, control) .Call("wrap__changeforest_api", X, method, segmentation, control, PACKAGE="changeforest")
